import React from 'react'
import * as monaco from 'monaco-editor'
import suggestions from './sqlDoc'
import { api } from 'xadmin'
import { Icon, C } from 'xadmin-ui';
import pythonClient from './pythonClient';
// Since packaging is done by you, you need
// to instruct the editor how you named the
// bundles that contain the web workers.
self.MonacoEnvironment = {
  getWorkerUrl: function (moduleId, label) {
    if (label === 'json') {
      return '/editor/json.worker.js';
    }
    if (label === 'css') {
      return '/editor/css.worker.js';
    }
    if (label === 'html') {
      return '/editor/html.worker.js';
    }
    if (label === 'typescript' || label === 'javascript') {
      return '/editor/ts.worker.js';
    }
    return '/editor/editor.worker.js';
  }
}
function processSize(size) {
  return !/^\d+$/.test(size.toString()) ? size : `${size}px`;
}

const VariableSelect = (props) => {

  const { bindVariable, onChange } = props

  const [ visible, setVisible ] = React.useState(false)
  const [ value, setValue ] = React.useState(null)
  const domRef = React.useRef()

  const onModalCancel = (value) => {
    onChange(value)
    // item.value = value
    // onChange(formatObj(treeData[0]))
    // setExpandedKeys(pre => _.cloneDeep(pre))
    setVisible(false)
  }

  React.useEffect(() => {
    setValue(null)
  }, [ visible ])

  return <>
    <div ref={domRef} style={{ display: 'inline-block', width: 30, height: 30, marginLeft: 10}} onMouseDown={e => e.preventDefault()} onClick={() => { setVisible(true) }}>
      <Icon
        onMouseDown={e => e.preventDefault()}
        svg={require('../icons/数据绑定.svg')}
      />
    </div>
    <C 
      is='CodeEditor.VariableEditor'
      isModal={true}
      modalVisible={visible}
      onModalCancel={onModalCancel}
      schema={_.cloneDeep(bindVariable)}
      stringify={true}
      value={value}
      onChange={setValue}
    />
  </> 
}


const CodeEditor = (props, ref) => {

  const { width, height, value, defaultValue, onChange, theme, options,
    language, overrideServices, editorWillMount, editorDidMount, onHeightChange, disableEnter, bindVariable } = props

  const container = React.useRef()
  const editor = React.useRef()
  const trigger = React.useRef(false)
  const onChangeRef = React.useRef(onChange)

  const fixedWidth = processSize(width);
  const fixedHeight = processSize(height);

  const style = {
    width: fixedWidth,
    height: fixedHeight,
  };

  React.useLayoutEffect(() => {
    if (container.current) {
      let opts = { ...options };
      if (editorWillMount) {
        opts = { ...opts, ...editorWillMount(monaco) }
      }
      const val = value != null && _.isString(value) ? value : defaultValue
      editor.current = monaco.editor.create(
        container.current,
        {
          value: val,
          language,
          glyphMargin: true,
          lightbulb: {
            enabled: true
          },
          ...opts,
          ...(theme ? { theme } : {}),
        },
        overrideServices
      );
      if (editorDidMount) {
        editorDidMount(editor.current, monaco);
      }
      const subscription = editor.current.onDidChangeModelContent((event) => {
        if (onChange && !trigger.current) {
          onChangeRef.current(editor.current.getValue(), event);
        }
        const lineHeight = document.querySelector('.view-lines')?.offsetHeight
        onHeightChange && lineHeight && onHeightChange(lineHeight)
      })
      // 解决 antd 的 Modal 动画问题
      setTimeout(() => {
        editor.current?.layout()
      })

      return () => {
        if (editor.current) {
          editor.current.dispose();
          const model = editor.current.getModel();
          if (model) {
            model.dispose();
          }
        }
        if (subscription) {
          subscription.dispose();
        }
      }
    }
  }, [])

  React.useEffect(() => {
    onChangeRef.current = onChange
  }, [onChange])

  React.useEffect(() => {
    if (!editor.current || !_.isString(value)) return

    const model = editor.current.getModel();
    if (value != null && value !== model.getValue()) {
      trigger.current = true;
      editor.current.pushUndoStop();

      model.pushEditOperations(
        [],
        [
          {
            range: model.getFullModelRange(),
            text: value,
          },
        ]
      );
      editor.current.pushUndoStop();
      trigger.current = false;
    }
  }, [value])

  React.useEffect(() => {
    if (!editor.current || !options) return

    const { model: _model, ...optionsWithoutModel } = options;
    editor.current.updateOptions(optionsWithoutModel);
  }, [options])

  React.useEffect(() => {
    if (!editor.current) return
    monaco.editor.setModelLanguage(editor.current.getModel(), language);
    if (language == 'sql') monaco.languages.registerCompletionItemProvider('sql', {
      triggerCharacters: ['\s', ',', '\n'],
      provideCompletionItems: () => {
        return { suggestions };
      }
    });
    if (language == 'javascript') {

      const librariesToFetch = [
        '/static/scriptTs/lodash.d.ts',
        '/static/scriptTs/moment.d.ts',
        '/static/scriptTs/crypto-js.d.ts',
        '/static/scriptTs/buffer.d.ts',
        '/static/scriptTs/xml-js.d.ts',
        '/static/scriptTs/formulajs.d.ts',
        '/static/scriptTs/logger.d.ts',
        '/static/scriptTs/apiLib.d.ts',
        '/static/scriptTs/api.d.ts'
      ];

      const fetchPromises = librariesToFetch.map((url) => {
        return api({ name: '' }).fetch(url).then(({ body }) => body);
      });

      Promise.all(fetchPromises)
        .then((librarySources) => {
          const [lodashLibSource, momentLibSource, cryptoLibSource, bufferLibSource, xmljsLibSource,
            formulajsLibSource, loggerLibSource, aipLibLibSource] = librarySources;
          monaco.languages.typescript.javascriptDefaults.setExtraLibs([
            { content: lodashLibSource, filePath: "lodash.d.ts" },
            { content: momentLibSource, filePath: "moment.d.ts" },
            { content: cryptoLibSource, filePath: "crypto-js.d.ts" },
            { content: bufferLibSource, filePath: "buffer.d.ts" },
            { content: xmljsLibSource, filePath: "xml-js.d.ts" },
            { content: formulajsLibSource, filePath: "formulajs.d.ts" },
            { content: loggerLibSource, filePath: "logger.d.ts" },
            { content: aipLibLibSource, filePath: "aipLib.d.ts" },
          ])
        })
        .catch((err) => {
          console.error('setExtraLibs err: ', err)
        });
    }
    let pyClient = null
    if (language === 'python') {
      pyClient = pythonClient(editor)
    }
    return () => {
      if (pyClient?.dispose) {
        pyClient.dispose()
      }
    }
  }, [language])

  React.useEffect(() => {
    monaco.editor.setTheme(theme);
  }, [theme])

  React.useEffect(() => {
    if (!editor.current) return
    editor.current.layout();
  }, [height, width])

  React.useEffect(() => {
    editor?.current?.onKeyDown((event) => {
      if (event.code === 'Enter' && disableEnter) {
        event.preventDefault()
      }
    })
  }, [disableEnter])

  const insert = (value) => {
    const position = editor?.current?.getPosition()
    if (position) {
      const editOperation = {
          range: new monaco.Range(position.lineNumber, position.column, position.lineNumber, position.column),
          text: value,
          forceMoveMarkers: true
      };
      editor?.current?.executeEdits('insert', [editOperation]);
    }
  }

  React.useImperativeHandle(ref, () => ({
    insert
  }))

  return <div style={{ position: 'relative' }}>
    {
      bindVariable ? <div style={{ position: 'absolute', zIndex: 10, cursor: 'pointer' }} title="绑定变量">
      <VariableSelect bindVariable={bindVariable} onChange={insert}></VariableSelect>
    </div> : null
    }
    <div
      ref={container}
      style={style}
      className="monaco-editor-container"
    />
  </div>
}

export default React.forwardRef(CodeEditor)
