/**
 * Input Field
 *
 * This can be considered as a free entry field which contains logic about the type of its content.
 *
 * - onChange: Returns the latest part of the expression rendered by this component. Used by its
 * parent (and grand parents) to complete structure, part by part.
 *
 * - value: Value of the field, passed from its parent.
 *
 * - type: Type of the field, will be determined with respect to the `typeof` result of the value.
 */

// Core
import React from 'react'
import PropTypes from 'prop-types'
import { Select, Input as AntInput, InputNumber, Switch, DatePicker } from 'antd'

// Helpers
import _ from 'lodash'

// Constants
const INPUT_TYPES = ['number', 'text', 'boolean', 'date', 'time'] // @TODO: Add Date!
const INPUT_TYPES_LABEL = [_r('数字'), _r('文本'), _r('布尔'), _r('日期'), _r('时间')]

function isDate(str) {
  // 使用 Moment.js 尝试解析日期
  return moment(str, 'YYYY-MM-DD', true).isValid();
}

function isTime(str) {
  // 使用 Moment.js 尝试解析时间
  return moment(str, 'YYYY-MM-DD HH:mm:ss', true).isValid();
}

// Helpers
const isNumeric = value => typeof value === 'number'
const getType = (value) => {
  console.log(value, 'value')
  if (isNumeric(value) || value == undefined || value == null) {
    return 'number'
  } else if (_.isBoolean(value)) {
    return 'boolean'
  } else if (isDate(value)) {
    return 'date'
  } else if (isTime(value)) {
    return 'time'
  } else {
    return 'text'
  }
}

// PropTypes
const { any, string, func } = PropTypes
const propTypes = {
  name: string,
  onChange: func.isRequired,
  value: any,
  type: string
}

const defaultProps = {
  name: '',
  type: INPUT_TYPES[0],
  value: ''
}

const TypeInput = ({ type, ...prop }) => {

  const onDateChange = (date, dateString) => {
    prop.onChange(dateString)
  }
  const onInputChange = (e) => {
    let value = e?.target?.value
    prop.onChange(value)
  }

  const onNumberChange = (e) => {
    let value = e
    prop.onChange(value)
  }

  if (type === 'number') {
    return <InputNumber {...prop} onChange={onNumberChange} />
  } else if (type === 'text') {
    return <AntInput {...prop} onChange={onInputChange} />
  } else if (type === 'boolean') {
    return <Switch checked={prop.value} {...prop} style={{ width: 50 }} />
  } else if (type === 'date') {
    return <DatePicker {...prop} value={prop.value ? moment(prop.value) : null} onChange={onDateChange} />
  } else if (type === 'time') {
    return <DatePicker {...prop} value={prop.value ? moment(prop.value) : null} onChange={onDateChange} showTime />
  }
}

const Input = ({ name, value = [], onChange }) => {
  const [type, setType] = React.useState(getType(value))

  // React.useEffect(() => {
  //   const newType = getType(value)
  //   if (newType != type) {
  //     setType(newType)
  //   }

  // }, [JSON.stringify(value)])

  const onTypeChange = (type) => {
    onChange('')
    setType(type)
  }


  const prop = {
    style: { width: 200 },
    name, value, onChange
  }
  return (
    <div style={{ display: 'flex' }}>
      <div style={{}}>
        <Select
          style={{ width: 100 }}
          allowClear={false}
          value={type || undefined}
          placeholder={_t1("请选择...")}
          onChange={onTypeChange}
        >
          {INPUT_TYPES.map((inputType, i) => (
            <Select.Option value={inputType}>{_t1(INPUT_TYPES_LABEL[i]) || inputType}</Select.Option>
          ))}
        </Select>
      </div>

      <div style={{ marginLeft: '.5rem' }}>
        <TypeInput type={type} {...prop} ></TypeInput>
      </div>
    </div>
  )
}

export default Input
